"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const generate_1 = __importDefault(require("nanoid/non-secure/generate"));
const response_1 = require("../util/response");
const moment_1 = __importDefault(require("moment"));
const lodash_1 = require("lodash");
const log_1 = require("../util/log");
exports.getPk = (resource) => lodash_1.pick(resource, 'pk', 'sk');
exports.createResourceHelper = ({ db }) => ({
    save: async ({ isNew, item }) => {
        const updatedAt = moment_1.default().unix();
        await db
            .put({
            Item: { ...item, updatedAt },
            ConditionExpression: isNew ? 'attribute_not_exists(pk) AND attribute_not_exists(sk)' : undefined
        })
            .catch(error => {
            log_1.out.info({ error });
            if (error.code === 'ConditionalCheckFailedException' && isNew) {
                throw response_1.response(response_1.BAD_REQUEST_400('Resource already exists'));
            }
            throw error;
        });
        return updatedAt;
    },
    get: async (key) => {
        const item = await db.get({
            Key: exports.getPk(key)
        });
        if (!item)
            throw response_1.response(response_1.NOT_FOUND_404);
        log_1.out.info({ item });
        return item;
    },
    update: async ({ pk, key, value }) => {
        const random = generate_1.default('abcdefghijklmnopqrstuvwxyz1234567890', 6);
        const updatedAt = await db.update({
            Key: pk,
            UpdateExpression: `SET ${key} = :${random}`,
            ExpressionAttributeValues: {
                [`:${random}`]: value
            }
        });
        return updatedAt;
    },
    updateList: async ({ pk, key, items }) => {
        const random = generate_1.default('abcdefghijklmnopqrstuvwxyz1234567890', 6);
        const updatedAt = await db.update({
            Key: pk,
            UpdateExpression: `SET ${key} = list_append(${key}, :${random})`,
            ExpressionAttributeValues: {
                [`:${random}`]: items
            }
        });
        return updatedAt;
    },
    delete: async ({ pk }) => {
        await db.delete({
            Key: exports.getPk(pk),
            ConditionExpression: 'attribute_exists(pk) AND attribute_exists(sk)'
        });
        return;
    }
});
