import { BaseResource } from '../types/Resource';
import { dbClient } from '../util/dbClient';
import { createEventHelper } from '../helpers/eventHelper';
import * as yup from 'yup';
interface Config<Data extends object & BaseResource> {
    db: ReturnType<typeof dbClient>;
    eventbridge: AWS.EventBridge;
    validationSchema: yup.ObjectSchema<Data>;
    hiddenKeys: Array<keyof Data>;
    ownerKeys: Array<keyof Data>;
}
export declare class Resource<Attributes extends object, Data extends Attributes & BaseResource = Attributes & BaseResource> {
    protected config: Config<Data>;
    protected eventHelper: ReturnType<typeof createEventHelper>;
    protected initial: Data;
    protected current: Data;
    constructor(params: Partial<Data> & {
        config: Pick<Config<Data>, 'db' | 'eventbridge'> & Partial<Pick<Config<Data>, 'hiddenKeys' | 'ownerKeys'>> & {
            validationSchema: yup.ObjectSchema<Attributes>;
        };
    });
    get data(): Data;
    set(data: Partial<Data>): void;
    get init(): Data;
    get owner(): Pick<Data, Exclude<keyof Data, keyof Data>>;
    get public(): Pick<Data, Exclude<keyof Data, keyof Data>>;
    get pk(): Pick<Data, "pk" | "sk">;
    save: (isNew?: boolean) => Promise<this>;
    create: () => Promise<this>;
    validate: () => Promise<Data>;
    delete: () => Promise<void>;
}
export {};
