"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Resource = void 0;
const lodash_1 = require("lodash");
const moment = require("moment");
const nanoid_1 = __importDefault(require("nanoid"));
const eventHelper_1 = require("../helpers/eventHelper");
const yup = __importStar(require("yup"));
class Resource {
    constructor(params) {
        this.save = async (isNew = false) => {
            const timestamp = moment().unix();
            this.current.updatedAt = timestamp;
            await this.validate();
            await this.config.db.put({
                Item: this.current
            }, isNew);
            return this;
        };
        this.create = async () => this.save(true);
        this.validate = async () => {
            await this.config.validationSchema.validate(this.current);
            return this.current;
        };
        this.delete = async () => {
            await this.config.db.delete({
                Key: this.pk
            });
            return;
        };
        const pk = params.pk || nanoid_1.default();
        const sk = params.sk || pk;
        const timestamp = moment().unix();
        const data = {
            pk,
            sk,
            resourceType: params.resourceType || 'Resource',
            createdAt: params.createdAt || timestamp,
            updatedAt: params.updatedAt || timestamp,
            ...lodash_1.omit(params, 'config')
        };
        this.initial = data;
        this.current = data;
        this.eventHelper = eventHelper_1.createEventHelper({ eventbridge: params.config.eventbridge });
        this.config = {
            db: params.config.db,
            eventbridge: params.config.eventbridge,
            hiddenKeys: params.config.hiddenKeys || [],
            ownerKeys: params.config.ownerKeys || [],
            validationSchema: yup
                .object({
                pk: yup.string().required(),
                sk: yup.string().required(),
                resourceType: yup.string().required(),
                createdAt: yup.number().positive().integer().required(),
                updatedAt: yup.number().positive().integer().required()
            })
                .concat(params.config.validationSchema)
        };
    }
    get data() {
        return this.current;
    }
    set(data) {
        this.current = { ...this.current, ...data };
        return;
    }
    get init() {
        return this.initial;
    }
    get owner() {
        return lodash_1.omit(this.current, this.config.hiddenKeys);
    }
    get public() {
        return lodash_1.omit(this.current, [...this.config.hiddenKeys, ...this.config.ownerKeys]);
    }
    get pk() {
        return lodash_1.pick(this.current, ['pk', 'sk']);
    }
}
exports.Resource = Resource;
